# -*- coding: utf-8 -*-
# moebel.py

from grafikfenster import *
from math import radians

from abc import ABC,abstractmethod # abstrakte Klasse realisieren

### ---------------------------------------------------
class Moebel(ABC):
    """Klasse Moebel
    Oberklasse für das Zeichnen und Bearbeiten von
    Möbel-Symbolen fuer den Raumplaner"""
    
    def __init__(self,
                 xPos,
                 yPos,
                 breite,
                 tiefe,
                 winkel,
                 farbe,
                 sichtbar):
        """Konstruktor mit vordefinierten Parametern
        ist bei Moebel nicht sinnvoll"""
        self.__x=xPos
        self.__y=yPos
        self.__b=breite
        self.__t=tiefe
        self.__w=winkel
        self.__f=farbe
        self.__s=sichtbar
        if sichtbar: self.Zeige()
        
    @abstractmethod
    def GibFigur(self):
        """definiert die zu zeichnende Figur (Pfad)"""
        return NotImplemented

    def Transformiere(self, path):
        '''transformiert den uebergebenen Pfad'''
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()

        x,y,b,t,w = self.__x, self.__y, self.__b, self.__t, self.__w
        gc.PushState()
        gc.Translate(x+b/2, y+t/2)
        gc.Rotate(radians(w))
        gc.Translate(-b/2, -t/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        return path

    def GibZeichenPfad(self):
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        return gc.CreatePath()

    def GibX(self):
        """Get-Methode fuer die x-Position"""
        return self.__x
    
    def GibY(self):
        """Get-Methode fuer die y-Position"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite"""
        return self.__b
    
    def GibTiefe(self):
        """Get-Methode fuer die Tiefe"""
        return self.__t
    
    def GibWinkel(self):
        """Get-Methode fuer den Winkel"""
        return self.__w
    
    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.__f

    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__s

    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Verberge()
        self.__x += weite
        self.Zeige()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Verberge()
        self.__y += weite
        self.Zeige()
    
    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Verberge()
        self.__w += winkel
        self.Zeige()

    def AendereFarbe(self, neueFarbe):
        """Veraendernde Methode fuer die Farbe"""
        sichtbar=self.GibSichtbar()
        if sichtbar: self.Verberge()
        self.__f = neueFarbe
        if sichtbar: self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__s = False
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__s = True
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung.
        Sollte nicht gehen! """
        try:
            moebel = Moebel(30, 30, 40, 40, 270, 'blue', True)
        except Exception as e:
            print( str(e) )

        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=True)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
